"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitOne = exports.emitMany = void 0;
const ArrayUtils = require("../utils/array_utils");
const Delimiters = require("../tokens/delimiters");
const Keywords = require("../tokens/keywords");
const ScopeTokens = require("../tokens/scope_tokens");
const MethodParameterTokens = require("../tokens/method_parameter_tokens");
function emitMany(writer, typeName, constructors, isFirst) {
    if (ArrayUtils.isFalsy(constructors))
        return;
    if (!isFirst)
        writer.writeLine();
    constructors.forEach((c, i, arr) => emitOne(writer, typeName, c, i === arr.length - 1));
}
exports.emitMany = emitMany;
function emitOne(writer, typeName, constructor, isLast) {
    emitDefinition(writer, typeName, constructor);
    emitBody(writer, constructor);
    emitEnd(writer, isLast);
}
exports.emitOne = emitOne;
function emitDefinition(writer, typeName, constructor) {
    const { access, isStatic, parameters } = constructor;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const staticTokens = isStatic ? [Keywords._static, Delimiters.space] : [];
    const methodParameterTokens = MethodParameterTokens.emit(parameters);
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...staticTokens)
        .write(typeName)
        .write(...methodParameterTokens)
        .writeLineNoTabs()
        .writeLine(ScopeTokens.open)
        .indent();
}
function emitBody(writer, constructor) {
    const { body } = constructor;
    body.statements.forEach(s => writer.writeLine(s));
}
function emitEnd(writer, isLast) {
    writer
        .unindent()
        .writeLine(ScopeTokens.close);
    if (!isLast)
        writer.writeLine();
}
