"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emit = void 0;
const ArrayUtils = require("../utils/array_utils");
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const Signs = require("../tokens/signs");
const ScopeTokens = require("../tokens/scope_tokens");
const AttributeEmitter = require("./attribute_emitter");
function emit(writer, _enum) {
    AttributeEmitter.emitMany(writer, _enum.attributes);
    emitDefinition(writer, _enum);
    emitMembers(writer, _enum.members);
    emitEnd(writer);
}
exports.emit = emit;
function emitDefinition(writer, _enum) {
    const { access, isNew, name } = _enum;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const newTokens = isNew ? [Keywords._new, Delimiters.space] : [];
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...newTokens)
        .write(Keywords._enum, Delimiters.space, name)
        .writeLineNoTabs()
        .writeLine(ScopeTokens.open)
        .indent();
}
function emitMembers(writer, members) {
    if (ArrayUtils.isFalsy(members))
        return;
    members.forEach((m, i, arr) => emitMember(writer, m, i === arr.length - 1));
}
function emitMember(writer, member, isLast) {
    const { attributes, name, value } = member;
    const valueTokens = value ? [Delimiters.space, Signs.equal, Delimiters.space, value] : [];
    const endTokens = isLast ? [] : [Delimiters.comma];
    AttributeEmitter.emitMany(writer, attributes);
    writer
        .outputTabs()
        .write(name)
        .write(...valueTokens)
        .write(...endTokens)
        .writeLineNoTabs();
}
function emitEnd(writer) {
    writer
        .unindent()
        .writeLine(ScopeTokens.close);
}
