"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitOne = exports.emitMany = void 0;
const ArrayUtils = require("../utils/array_utils");
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const Signs = require("../tokens/signs");
const TypeReferenceTokens = require("../tokens/type_reference_tokens");
const AttributeEmitter = require("./attribute_emitter");
function emitMany(writer, fields, isFirst) {
    if (ArrayUtils.isFalsy(fields))
        return;
    if (!isFirst)
        writer.writeLine();
    fields.forEach((f, i, arr) => emitOne(writer, f, i === arr.length - 1));
}
exports.emitMany = emitMany;
function emitOne(writer, field, isLast) {
    const { access, isNew, isStatic, assignment, type, name, value } = field;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const newTokens = isNew ? [Keywords._new, Delimiters.space] : [];
    const staticTokens = isStatic ? [Keywords._static, Delimiters.space] : [];
    const assignmentTokens = assignment ? [assignment, Delimiters.space] : [];
    const resultTokens = [...TypeReferenceTokens.emit(type), Delimiters.space];
    const valueTokens = value != null ? [Delimiters.space, Signs.equal, Delimiters.space, value] : [];
    emitAttributes(writer, field);
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...newTokens)
        .write(...staticTokens)
        .write(...assignmentTokens)
        .write(...resultTokens)
        .write(name)
        .write(...valueTokens)
        .write(Delimiters.semicolon)
        .writeLineNoTabs();
    if (!isLast)
        writer.writeLine();
}
exports.emitOne = emitOne;
function emitAttributes(writer, field) {
    const { attributes } = field;
    AttributeEmitter.emitMany(writer, attributes);
}
