"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitOne = exports.emitMany = void 0;
const ArrayUtils = require("../utils/array_utils");
const Delimiters = require("../tokens/delimiters");
const ScopeTokens = require("../tokens/scope_tokens");
const GenericTokens = require("../tokens/generic_tokens");
const TypeReferenceTokens = require("../tokens/type_reference_tokens");
const MethodParameterTokens = require("../tokens/method_parameter_tokens");
function emitMany(writer, methods, isFirst) {
    if (ArrayUtils.isFalsy(methods))
        return;
    if (!isFirst)
        writer.writeLine();
    methods.forEach((m, i, arr) => emitOne(writer, m, i === arr.length - 1));
}
exports.emitMany = emitMany;
function emitOne(writer, method, isLast) {
    emitDefinition(writer, method);
    emitBody(writer, method);
    emitEnd(writer, isLast);
}
exports.emitOne = emitOne;
function emitDefinition(writer, method) {
    const { access, modifier, result, name, generics, parameters } = method;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const modifierTokens = modifier ? [modifier, Delimiters.space] : [];
    const resultTokens = TypeReferenceTokens.emit(result);
    const genericTokens = GenericTokens.emit(generics);
    const methodParameterTokens = MethodParameterTokens.emit(parameters);
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...modifierTokens)
        .write(...resultTokens)
        .write(Delimiters.space)
        .write(name)
        .write(...genericTokens)
        .write(...methodParameterTokens)
        .writeLineNoTabs()
        .writeLine(ScopeTokens.open)
        .indent();
}
function emitBody(writer, method) {
    const { body } = method;
    body.statements.forEach(s => writer.writeLine(s));
}
function emitEnd(writer, isLast) {
    writer
        .unindent()
        .writeLine(ScopeTokens.close);
    if (!isLast)
        writer.writeLine();
}
