"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitOne = exports.emitMany = void 0;
const ArrayUtils = require("../utils/array_utils");
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const NamespaceTokens = require("../tokens/namespace_tokens");
const ScopeTokens = require("../tokens/scope_tokens");
const NamespaceImportEmitter = require("./namespace_import_emitter");
const TypeEmitter = require("./type_emitter");
function emitMany(writer, namespaces) {
    if (ArrayUtils.isFalsy(namespaces))
        return;
    namespaces.forEach(ns => emitOne(writer, ns));
}
exports.emitMany = emitMany;
function emitOne(writer, namespace) {
    const { path, imports, types } = namespace;
    emitDefinition(writer, path);
    NamespaceImportEmitter.emitMany(writer, imports);
    TypeEmitter.emitMany(writer, types);
    emitEnd(writer);
}
exports.emitOne = emitOne;
function emitDefinition(writer, path) {
    writer
        .writeLine(Keywords.namespace, Delimiters.space, ...NamespaceTokens.emit(path))
        .writeLine(ScopeTokens.open)
        .indent();
}
function emitEnd(writer) {
    writer
        .unindent()
        .writeLine(ScopeTokens.close)
        .writeLine();
}
