"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitOne = exports.emitMany = void 0;
const ArrayUtils = require("../utils/array_utils");
const Signs = require("../tokens/signs");
const Delimiters = require("../tokens/delimiters");
const ScopeTokens = require("../tokens/scope_tokens");
const PropertyTokens = require("../tokens/property_tokens");
const TypeReferenceTokens = require("../tokens/type_reference_tokens");
const AttributeEmitter = require("./attribute_emitter");
function emitMany(writer, properties, isFirst) {
    if (ArrayUtils.isFalsy(properties))
        return;
    if (!isFirst)
        writer.writeLine();
    properties.forEach((p, i, arr) => emitOne(writer, p, i === arr.length - 1));
}
exports.emitMany = emitMany;
function emitOne(writer, property, isLast) {
    const { access, type, name, value } = property;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const resultTokens = TypeReferenceTokens.emit(type);
    const bodyTokens = emitBody();
    const valueTokens = value != null ? [Delimiters.space, Signs.equal, Delimiters.space, value, Delimiters.semicolon] : [];
    emitAttributes(writer, property);
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...resultTokens)
        .write(Delimiters.space)
        .write(name)
        .write(Delimiters.space)
        .write(...bodyTokens)
        .write(...valueTokens)
        .writeLineNoTabs();
    if (!isLast)
        writer.writeLine();
}
exports.emitOne = emitOne;
function emitAttributes(writer, property) {
    const { attributes } = property;
    AttributeEmitter.emitMany(writer, attributes);
}
function emitBody() {
    return [
        ScopeTokens.open, Delimiters.space,
        ...emitGetter(), Delimiters.space,
        ...emitterSetter(), Delimiters.space,
        ScopeTokens.close
    ];
}
function emitGetter() {
    return [PropertyTokens.get, Delimiters.semicolon];
}
function emitterSetter() {
    return [PropertyTokens.set, Delimiters.semicolon];
}
